@@ -1,4 +1,5 @@
 import argparse
+
 from pyhocon import ConfigFactory, ConfigTree
 
 
@@ -41,7 +42,25 @@ def parse_args():
     return args
 
 
-def parse_config(args):
+def get_resource_file_content(resource_file_name, config_package):
+    try:
+        # python 3.7+
+        import importlib.resources
+        with importlib.resources.open_text(config_package, f"{resource_file_name}") as file:
+            data = file.read()
+    except:
+        # python < 3.7
+        import importlib.util
+        config_instances_path = importlib.util.find_spec(config_package).submodule_search_locations[0]
+        config_file_path = f"{config_instances_path}/{resource_file_name}"
+        with open(config_file_path, 'r') as file:
+            data = file.read()
+    return data
+
+
+# 1 arg parse_config() for backward compatibility.
+def parse_config(args, pipeline_dot_conf=None, config_package=None):
+    config_package = "prophecy_config_instances" if config_package is None else config_package
     if args.file is not None:
         if hasattr(args, "defaultConfFile"):
             default_config = ConfigFactory.parse_file(
@@ -54,7 +73,7 @@ def parse_config(args):
             # python 3.7+
             import importlib.resources
             with importlib.resources.open_text(
-                    "prophecy_config_instances",
+                    config_package,
                     "{instance}.json".format(instance=args.confInstance),
             ) as file:
                 data = file.read()
@@ -62,7 +81,7 @@ def parse_config(args):
         except:
             # python < 3.7
             import importlib.util
-            config_instances_path = importlib.util.find_spec("prophecy_config_instances").submodule_search_locations[0]
+            config_instances_path = importlib.util.find_spec(config_package).submodule_search_locations[0]
             config_file_path = f"{config_instances_path}/{args.confInstance}.json"
             with open(config_file_path, 'r') as file:
                 data = file.read()
@@ -79,4 +98,17 @@ def parse_config(args):
             c = config.split("=", 1)
             conf.put(c[0], c[1])
 
+    if pipeline_dot_conf is not None:
+        # `resolve=False` is important here because variable substitution should happen after overriding
+        try:
+            # Check in resources/
+            pipeline_default_conf = ConfigFactory.parse_string(get_resource_file_content(pipeline_dot_conf, config_package),
+                                                               resolve=False)
+        except:
+            # Check as full file path
+            pipeline_default_conf = ConfigFactory.parse_file(pipeline_dot_conf, resolve=False)
+        # `resolve=True` so that variables get substituted at this step with overridden values
+        # pipeline_dot_conf contains default values for a given pipeline. It should have the lowest priority,
+        conf = conf.with_fallback(pipeline_default_conf, resolve=True)
+
     return conf
